package javax.swing.plaf.synth;

import javax.swing.plaf.basic.BasicHTML;
import java.beans.PropertyChangeEvent;
import java.awt.FontMetrics;
import java.awt.Font;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.Rectangle;
import javax.swing.text.View;
import java.awt.Graphics;
import javax.swing.JToolTip;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import java.beans.PropertyChangeListener;
import javax.swing.plaf.basic.BasicToolTipUI;

public class SynthToolTipUI extends BasicToolTipUI implements PropertyChangeListener, SynthUI
{
    private SynthStyle style;
    
    public SynthToolTipUI() {
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new SynthToolTipUI();
    }
    
    @Override
    protected void installDefaults(JComponent component) {
        this.updateStyle(component);
    }
    
    private void updateStyle(JComponent component) {
        SynthContext context = this.getContext(component, 1);
        this.style = SynthLookAndFeel.updateStyle(context, this);
        context.dispose();
    }
    
    @Override
    protected void uninstallDefaults(JComponent component) {
        SynthContext context = this.getContext(component, 1);
        this.style.uninstallDefaults(context);
        context.dispose();
        this.style = null;
    }
    
    @Override
    protected void installListeners(JComponent component) {
        component.addPropertyChangeListener(this);
    }
    
    @Override
    protected void uninstallListeners(JComponent component) {
        component.removePropertyChangeListener(this);
    }
    
    @Override
    public SynthContext getContext(JComponent component) {
        return this.getContext(component, this.getComponentState(component));
    }
    
    private SynthContext getContext(JComponent component, int n) {
        return SynthContext.getContext(component, this.style, n);
    }
    
    private int getComponentState(JComponent component) {
        JComponent component2 = ((JToolTip)component).getComponent();
        if (component2 != null && !component2.isEnabled()) {
            return 8;
        }
        return SynthLookAndFeel.getComponentState(component);
    }
    
    @Override
    public void update(Graphics graphics, JComponent component) {
        SynthContext context = this.getContext(component);
        SynthLookAndFeel.update(context, graphics);
        context.getPainter().paintToolTipBackground(context, graphics, 0, 0, component.getWidth(), component.getHeight());
        this.paint(context, graphics);
        context.dispose();
    }
    
    @Override
    public void paintBorder(SynthContext synthContext, Graphics graphics, int n, int n2, int n3, int n4) {
        synthContext.getPainter().paintToolTipBorder(synthContext, graphics, n, n2, n3, n4);
    }
    
    @Override
    public void paint(Graphics graphics, JComponent component) {
        SynthContext context = this.getContext(component);
        this.paint(context, graphics);
        context.dispose();
    }
    
    protected void paint(SynthContext synthContext, Graphics graphics) {
        JToolTip toolTip = (JToolTip)synthContext.getComponent();
        Insets insets = toolTip.getInsets();
        View view = (View)toolTip.getClientProperty("html");
        if (view != null) {
            view.paint(graphics, new Rectangle(insets.left, insets.top, toolTip.getWidth() - (insets.left + insets.right), toolTip.getHeight() - (insets.top + insets.bottom)));
        }
        else {
            graphics.setColor(synthContext.getStyle().getColor(synthContext, ColorType.TEXT_FOREGROUND));
            graphics.setFont(this.style.getFont(synthContext));
            synthContext.getStyle().getGraphicsUtils(synthContext).paintText(synthContext, graphics, toolTip.getTipText(), insets.left, insets.top, -1);
        }
    }
    
    @Override
    public Dimension getPreferredSize(JComponent component) {
        SynthContext context = this.getContext(component);
        Insets insets = component.getInsets();
        Dimension dimension = new Dimension(insets.left + insets.right, insets.top + insets.bottom);
        String tipText = ((JToolTip)component).getTipText();
        if (tipText != null) {
            View view = (component != null) ? ((View)component.getClientProperty("html")) : null;
            if (view != null) {
                dimension.width += (int)view.getPreferredSpan(0);
                dimension.height += (int)view.getPreferredSpan(1);
            }
            else {
                Font font = context.getStyle().getFont(context);
                FontMetrics fontMetrics = component.getFontMetrics(font);
                dimension.width += context.getStyle().getGraphicsUtils(context).computeStringWidth(context, font, fontMetrics, tipText);
                dimension.height += fontMetrics.getHeight();
            }
        }
        context.dispose();
        return dimension;
    }
    
    @Override
    public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
        if (SynthLookAndFeel.shouldUpdateStyle(propertyChangeEvent)) {
            this.updateStyle((JToolTip)propertyChangeEvent.getSource());
        }
        String propertyName = propertyChangeEvent.getPropertyName();
        if (propertyName.equals("tiptext") || "font".equals(propertyName) || "foreground".equals(propertyName)) {
            JToolTip c = (JToolTip)propertyChangeEvent.getSource();
            BasicHTML.updateRenderer(c, c.getTipText());
        }
    }
}
